@@ -151,41 +151,6 @@ exit_handler(int sig UNUSED)
 }
 
 
-/*
- * The logging callback for libevent.  We hook this into our message system so
- * that libevent messages are handled the same way as our other internal
- * messages.  This function should be passed to event_set_log_callback at the
- * start of libevent initialization.
- */
-static void
-event_log_callback(int severity, const char *message)
-{
-    switch (severity) {
-    case EVENT_LOG_DEBUG:
-        debug("%s", message);
-        break;
-    case EVENT_LOG_MSG:
-        notice("%s", message);
-        break;
-    default:
-        warn("%s", message);
-        break;
-    }
-}
-
-
-/*
- * The fatal callback for libevent.  Convert this to die, so that it's logged
- * the same as our other messages.  This function should be passed to
- * event_set_fatal_callback at the start of libevent initialization.
- */
-static void
-event_fatal_callback(int err)
-{
-    die("fatal libevent error (%d)", err);
-}
-
-
 /*
  * Given a service name, imports it and acquires credentials for it, storing
  * them in the second argument.  Returns true on success and false on failure,
@@ -559,8 +524,8 @@ main(int argc, char *argv[])
     message_program_name = "remctld";
 
     /* Initialize the logging and fatal callbacks for libevent. */
-    event_set_log_callback(event_log_callback);
-    event_set_fatal_callback(event_fatal_callback);
+    event_set_log_callback(server_event_log_callback);
+    event_set_fatal_callback(server_event_fatal_callback);
 
     /* Initialize options. */
     memset(&options, 0, sizeof(options));
