@@ -1,8 +1,8 @@
 /*
  * Test suite for the client connection negotiation code.
  *
- * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2006, 2007, 2009, 2010, 2012
+ * Written by Russ Allbery <eagle@eyrie.org>
+ * Copyright 2006, 2007, 2009, 2010, 2012, 2014
  *     The Board of Trustees of the Leland Stanford Junior University
  *
  * See LICENSE for licensing terms.
@@ -30,17 +30,6 @@
 #include <util/tokens.h>
 
 
-/*
- * The die handler function set by accept_connection so that it will exit with
- * _exit and not exit on errors and not run the cleanup functions.
- */
-static int
-exit_child(void)
-{
-    _exit(1);
-}
-
-
 /*
  * Create a socket, accept a single connection, try to establish a context,
  * and then close the connection and exit.  We run this in a subprocess to
@@ -50,7 +39,7 @@ exit_child(void)
  * version 1 and then goes back to version 2.
  */
 static void
-accept_connection(char *pidfile, int protocol)
+accept_connection(const char *pidfile, int protocol)
 {
     struct sockaddr_in saddr;
     socket_type s, conn;
@@ -64,9 +53,6 @@ accept_connection(char *pidfile, int protocol)
     gss_name_t client;
     gss_OID doid;
 
-    /* Set up the exit handler so that we don't call exit. */
-    message_fatal_cleanup = exit_child;
-
     /* Create the socket and accept the connection. */
     saddr.sin_family = AF_INET;
     saddr.sin_port = htons(14373);
@@ -117,10 +103,11 @@ accept_connection(char *pidfile, int protocol)
         }
     } while (major == GSS_S_CONTINUE_NEEDED);
 
-    /* All done.  Don't bother cleaning up memory, just exit. */
+    /* All done.  Clean up memory. */
+    gss_release_name(&minor, &client);
+    gss_delete_sec_context(&minor, &context, GSS_C_NO_BUFFER);
     socket_close(conn);
     socket_close(s);
-    _exit(0);
 }
 
 
@@ -174,12 +161,15 @@ main(void)
     path = test_tmpdir();
     basprintf(&pidfile, "%s/pid", path);
     for (protocol = 0; protocol <= 2; protocol++) {
-        r = remctl_new();
         child = fork();
         if (child < 0)
             sysbail("cannot fork");
-        else if (child == 0)
+        else if (child == 0) {
+            test_tmpdir_free(path);
             accept_connection(pidfile, protocol);
+            free(pidfile);
+            exit(0);
+        }
         alarm(1);
         while (access(pidfile, F_OK) < 0) {
             tv.tv_sec = 0;
@@ -187,6 +177,7 @@ main(void)
             select(0, NULL, NULL, NULL, &tv);
         }
         alarm(0);
+        r = remctl_new();
         if (!remctl_open(r, "127.0.0.1", 14373, config->principal)) {
             notice("# open error: %s", remctl_error(r));
             ok_block(5, 0, "protocol %d", protocol);
