@@ -4,7 +4,7 @@
  * The canonical version of this file is maintained in the rra-c-util package,
  * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
- * Written by Russ Allbery <rra@stanford.edu>
+ * Written by Russ Allbery <eagle@eyrie.org>
  *
  * The authors hereby relinquish any claim to any copyright that they may have
  * in this work, whether granted under contract or by operation of law or
@@ -21,7 +21,7 @@
 
 #include <tests/tap/basic.h>
 
-const char *test_inet_ntoa(const struct in_addr);
+char *test_inet_ntoa(struct in_addr);
 
 
 static void
