@@ -21,7 +21,8 @@ int
 main(void)
 {
     struct rule rule = {
-        NULL, 0, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0, NULL, NULL, NULL
+        NULL, 0, NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL, 0, 0, NULL,
+        NULL, NULL
     };
     struct iovec **command;
     int i;
