@@ -5,6 +5,7 @@
  * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
+ * Copyright 2014, 2016 Russ Allbery <eagle@eyrie.org>
  * Copyright 2009, 2010, 2011, 2012, 2013
  *     The Board of Trustees of the Leland Stanford Junior University
  * Copyright (c) 2004, 2005, 2006, 2007, 2008, 2010
@@ -45,9 +46,9 @@ BEGIN_DECLS
 
 /*
  * Create a socket of the given type and bind it to the specified address and
- * port (either IPv4 or IPv6), returning the resulting file descriptor or -1
- * on error.  Errors are reported using warn/syswarn.  To bind to all
- * interfaces, use "any" or "all" for address.
+ * port (either IPv4 or IPv6), returning the resulting file descriptor or
+ * INVALID_SOCKET on error.  Errors are reported using warn/syswarn.  To bind
+ * to all interfaces, use "any" or "all" for address.
  */
 socket_type network_bind_ipv4(int type, const char *addr, unsigned short port)
     __attribute__((__nonnull__));
@@ -73,7 +74,7 @@ void network_bind_all_free(socket_type *fds);
 /*
  * Wait on an array of file descriptor for one of them to select ready for
  * read, and return the first file descriptor that does so.  This is primarily
- * intended for UDP services listening on mutliple file descriptors.  TCP
+ * intended for UDP services listening on multiple file descriptors.  TCP
  * services will probably want to use network_accept_any instead.
  *
  * This is not intended to be a replacement for a full event loop, just some
@@ -97,8 +98,10 @@ socket_type network_accept_any(socket_type fds[], unsigned int count,
 /*
  * Create a socket and connect it to the remote service given by the linked
  * list of addrinfo structs.  Returns the new file descriptor on success and
- * -1 on failure, with the error left in errno.  Takes an optional source
- * address and a timeout in seconds, which may be 0 for no timeout.
+ * INVALID_SOCKET on failure, with the error left in errno.  Takes an optional
+ * source address and a timeout in seconds, which may be 0 for no timeout.
+ * (Source may also be "all" or "any", which mean the same thing as NULL: do
+ * not use any particular source address.)
  */
 socket_type network_connect(const struct addrinfo *, const char *source,
                             time_t)
@@ -115,14 +118,30 @@ socket_type network_connect_host(const char *host, unsigned short port,
 /*
  * Creates a socket of the specified domain and type and binds it to the
  * appropriate source address, either the one supplied or all addresses if the
- * source address is NULL or "all".  Returns the newly created file descriptor
- * or -1 on error.
+ * source address is NULL, "all", or "any".  Returns the newly created file
+ * descriptor or INVALID_SOCKET on error.
  *
  * This is a lower-level function intended primarily for the use of clients
  * that will then go on to do a non-blocking connect.
  */
 socket_type network_client_create(int domain, int type, const char *source);
 
+/*
+ * Set various socket flags if possible, but do nothing, silently, if that
+ * option is not supported.  If the option is supported but setting the flag
+ * fails, log a warning with syswarn.
+ *
+ * network_set_freebind sets IP_FREEBIND, which allows binding IPv6 addresses
+ * that may not have been set up yet.  network_set_reuseaddr sets SO_REUSEADDR
+ * so that something new can listen on the same port immediately if the daemon
+ * dies unexpectedly.  network_set_v6only sets IP_V6ONLY, which avoids binding
+ * to the backward-compatibility IPv4 address when binding an IPv6 socket
+ * (generally preferred since the behavior is more predictable).
+ */
+void network_set_freebind(socket_type fd);
+void network_set_reuseaddr(socket_type fd);
+void network_set_v6only(socket_type fd);
+
 /*
  * Read or write the specified number of bytes to the network, enforcing a
  * timeout.  Both return true on success and false on failure; on failure, the
